# =====================================
# generator=datazen
# version=3.1.2
# hash=5f72b7e25844456d9d2f200300b91b3e
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A configuration-file management system."
PKG_NAME = "rcmpy"
VERSION = "0.2.0"
